'use strict';

// Code based on the JavaScript generator available at https://github.com/google/blockly

var SPARQL = new Blockly.Generator('SPARQL');
/**
 * List of illegal variable names.
 * This is not intended to be a security feature.  Blockly is 100% client-side,
 * so bypassing this list is trivial.  This is intended to prevent users from
 * accidentally clobbering a built-in object or function.
 * @private
 */
SPARQL.addReservedWords('');

/**
 * Initialise the database of variable names.
 * @param {!Blockly.Workspace} workspace Workspace to generate code from.
 */
SPARQL.init = function(workspace) {

};

/**
 * Prepend the generated code with the variable definitions.
 * @param {string} code Generated code.
 * @return {string} Completed code.
 */
SPARQL.finish = function(code) {
  return code;
};

/**
 * Naked values are top-level blocks with outputs that aren't plugged into
 * anything.  A trailing semicolon is needed to make this legal.
 * @param {string} line Line of generated code.
 * @return {string} Legal line of code.
 */
SPARQL.scrubNakedValue = function(line) {
  return line + ';\n';
};

/**
 * Encode a string as a properly escaped SPARQL string, complete with
 * quotes.
 * @param {string} string Text to encode.
 * @return {string} SPARQL string.
 * @private
 */
SPARQL.quote_ = function(string) {
  // TODO: This is a quick hack.  Replace with goog.string.quote
  string = string.replace(/\\/g, '\\\\')
                 .replace(/\n/g, '\\\n')
                 .replace(/'/g, '\\\'');
  return '\'' + string + '\'';
};

/**
 * Common tasks for generating SPARQL from blocks.
 * Handles comments for the specified block and any connected value blocks.
 * Calls any statements following this block.
 * @param {!Blockly.Block} block The current block.
 * @param {string} code The SPARQL code created for this block.
 * @return {string} SPARQL code with comments and subsequent blocks added.
 * @private
 */
SPARQL.scrub_ = function(block, code) {
  var commentCode = '';
  // Only collect comments for blocks that aren't inline.
  if (!block.outputConnection || !block.outputConnection.targetConnection) {
    // Collect comment for this block.
    var comment = block.getCommentText();
    if (comment) {
      commentCode += SPARQL.prefixLines(comment, '// ') + '\n';
    }
    // Collect comments for all value arguments.
    // Don't collect comments for nested statements.
    for (var x = 0; x < block.inputList.length; x++) {
      if (block.inputList[x].type == Blockly.INPUT_VALUE) {
        var childBlock = block.inputList[x].connection.targetBlock();
        if (childBlock) {
          var childComment = SPARQL.allNestedComments(childBlock);
          if (childComment) {
            commentCode += SPARQL.prefixLines(childComment, '// ');
          }
        }
      }
    }
  }
  var nextBlock = block.nextConnection && block.nextConnection.targetBlock();
  var nextCode = SPARQL.blockToCode(nextBlock);
  return commentCode + code + nextCode;
};

function isDisconnected(block) {
  return block.getParent() == undefined;
}

var functionCount = 0;
SPARQL.mapping = function(block) {
  if (!block) {
    return '';
  }
  functionCount = 0; 
  functions = ''; 
  return "prefix owl: <http://www.w3.org/2002/07/owl#> \n"+ SPARQL.statementToCode(block, 'vocabs') + '\n' + SPARQL.statementToCode(block, 'mappings') + '\n\n';
};

SPARQL.prefix = function(block) {
  if (isDisconnected(block)) {
    return '';
  }
  
  var prefix = block.getFieldValue('PREFIX');
  var uri = block.getFieldValue('URI');
  return "prefix "+ prefix +": <" + uri + "> \n";
};

SPARQL.base = function(block) {
  if (isDisconnected(block)) {
    return '';
  }
  
  var uri = block.getFieldValue('URI');
  return "base <" + uri + "> \n";
};

SPARQL.predefinedprefix = function(block) {
  if (isDisconnected(block)) {
    return '';
  }
  
  var prefix = block.getFieldValue('PREFIX');
  return "prefix "+ prefix +" \n";
};

var source = '';
SPARQL.simplemapping = function(block) {
  source = SPARQL.statementToCode(block, 'source');
  return '\n#Id ' + block.getFieldValue('ID') + '\n' + SPARQL.statementToCode(block, 'properties');
}

SPARQL.concept = function(block) {
  return block.getFieldValue('CONCEPT');
}

SPARQL.relation_concept = function(block) {
  var relation = SPARQL.statementToCode(block, 'relation').trim();
  if(relation == 'classeq') {
    return 'CONSTRUCT {?subj a ' + SPARQL.statementToCode(block, 'target') + ' . }' +
           '\n WHERE {?subj a ' + source + ' . }\n';
  } else if(relation == 'propeq') {
    return 'CONSTRUCT {?subj ' + SPARQL.statementToCode(block, 'target') + ' ?obj . }' +
           '\n WHERE {?subj ' + source + ' ?obj . }\n';
  } else if(relation == 'sameas') {
    return 'CONSTRUCT { ' + source + ' owl:sameAs ' + SPARQL.statementToCode(block, 'target') + ' . }' +
           '\n WHERE { }\n';
  } else if(relation == 'propinv') {
    return 'CONSTRUCT {?subj ' + SPARQL.statementToCode(block, 'target') + ' ?obj . }' +
           '\n WHERE {?obj ' + source + ' ?subj . }\n';
  } else if(relation == 'subclass') {
    return 'CONSTRUCT {' + source + ' rdfs:subClassOf ' + SPARQL.statementToCode(block, 'target') + ' . }' +
           '\n WHERE { }\n';
  } else if(relation == 'subprop') {
    return 'CONSTRUCT {' + source + ' rdfs:subPropertyOf ' + SPARQL.statementToCode(block, 'target') + ' . }' +
           '\n WHERE { }\n';
  }
  return 'CONSTRUCT {' + source + ' owl:disjointWith ' + SPARQL.statementToCode(block, 'target') + ' . }' +
           '\n WHERE { }\n';
}

SPARQL.relation = function(block){
  return block.getFieldValue('RELATION');
}

SPARQL.complexmapping = function(block) {
  var construct = SPARQL.statementToCode(block, 'target');
  var where = SPARQL.statementToCode(block, 'source');
  return '\n#Id' + block.getFieldValue('ID') + '\nCONSTRUCT { ' + construct + ' \n} \n  WHERE { ' + where + functions + '\n}';
}

var subject = '';
SPARQL.subject = function(block) {
  subject = SPARQL.statementToCode(block, 'subject');
  return SPARQL.statementToCode(block, 'predicate_object') ;
}

SPARQL.subjectdef = function(block) {
  return block.getFieldValue('SUBJECT');
}

SPARQL.predicate_object = function(block) {
  return '\n' + subject + SPARQL.statementToCode(block, 'predicate') + ' ' + SPARQL.statementToCode(block, 'object') + ' . ';
}

SPARQL.predicate = function(block){
  return block.getFieldValue('PREDICATE');
}

SPARQL.variables_get_param = function(block){
  return '';// block.getFieldValue('NAME');
}

SPARQL.variables_get = function(block){
  return '?' + block.getFieldValue('VAR');
}

SPARQL.object = function(block){
  var withOption = block.getFieldValue('WITHOPTION');
  var code = '';
  if(withOption == 'TRUE') {
    var option = block.getFieldValue('OPTIONTYPE');
    if(option == 'literal') {
      return '\"' + block.getFieldValue('OBJECT') + '\"';
    } else if(option == 'datatype') {
      return block.getFieldValue('OBJECT') + '^^' + block.getFieldValue('lang_datatype');
    } else {
      return block.getFieldValue('OBJECT') + '@'  + block.getFieldValue('lang_datatype');;
    }
  }
  return block.getFieldValue('OBJECT');
}

SPARQL.classes = function(block){
  var classes = '';
  for (var i = 0; i < this.arguments_.length; i++) {
     classes += subject + " a " + this.arguments_[i] + " .\n";
  }
  return classes;
}

SPARQL.class_list = function(block){
  return '';
}

SPARQL.class_each = function(block){
  return '';
}

SPARQL.bnode = function(block){
  functions += '   FILTER isBlank( '+ subject + ')';
  return '';
}

var functions = '';

SPARQL.concat = function(block){
  var args = '';
  for(var i = 0; i < block.arguments_.length; i++) {
    if(i != 0) {
      args += ', ';
    }
    if(block.arguments_[i][0] == 'variables_get_param') {
      args += '?' + block.arguments_[i][1];
    } else {
      args += '\"' + block.arguments_[i][1] + '\"';
    }
  }
  var isIRI = block.as == 'iri';
  if(isIRI){
    functions += '\nBIND ( URI (concat(' + args + ')) AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?  
  } else {
    functions += '\nBIND (concat(' + args + ') AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  }
  
  return '?result' + functionCount;
}

SPARQL.strbefore = function(block){
  var args = '';
  for(var i = 0; i < block.arguments_.length; i++) {
    if(i != 0) {
      args += ', ';
    }
    if(block.arguments_[i][0] == 'variables_get_param') {
      args += '?' + block.arguments_[i][1];
    } else {
      args += '\"' + block.arguments_[i][1] + '\"';
    }
  }
  var isIRI = block.as == 'iri';
  if(isIRI){
    functions += '\nBIND ( URI (STRBEFORE(' + args + ')) AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?  
  } else {
    functions += '\nBIND (STRBEFORE(' + args + ') AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  }
  
  return '?result' + functionCount;
}

SPARQL.strafter = function(block){
  var args = '';
  for(var i = 0; i < block.arguments_.length; i++) {
    if(i != 0) {
      args += ', ';
    }
    if(block.arguments_[i][0] == 'variables_get_param') {
      args += '?' + block.arguments_[i][1];
    } else {
      args += '\"' + block.arguments_[i][1] + '\"';
    }
  }
  var isIRI = block.as == 'iri';
  if(isIRI){
    functions += '\nBIND ( URI (STRAFTER(' + args + ')) AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?  
  } else {
    functions += '\nBIND (STRAFTER(' + args + ') AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  }
  
  return '?result' + functionCount;
}

SPARQL.minutesToSeconds = function(block) {
  var args = '';
  for(var i = 0; i < block.arguments_.length; i++) {
    if(i != 0) {
      args += ', ';
    }
    if(block.arguments_[i][0] == 'variables_get_param') {
      args += '?' + block.arguments_[i][1];
    } else {
      args += '\"' + block.arguments_[i][1] + '\"';
    }
  }
  var isFloat = block.as == 'float';
  if(isFloat){
    functions += '\nBIND ((xsd:float(' + args + ')/60) AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  } else {
    functions += '\nBIND ((' + args + '/60) AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  }
  
  return '?result' + functionCount;
}

SPARQL.percentageOf = function(block) {
  var args = '(' + getParam(block, 0) + '/' + getParam(block, 1) + ')*100' ;
  var isFloat = block.as == 'float';
  if(isFloat){
    functions += '\nBIND (xsd:float(' + args + ') AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  } else {
    functions += '\nBIND (' + args + ' AS ?result' +(++functionCount)+ ')'; // TODO check where function can be used?
  }
  
  return '?result' + functionCount;
}

function getParam(block, i){
  if(block.arguments_[i][0] == 'variables_get_param') {
    return '?' + block.arguments_[i][1];
  } 
  return '\"' + block.arguments_[i][1] + '\"';
}

SPARQL.parameter = function(block){
  return '';
}

SPARQL.parameters = function(block){
  return '';
}

